# frozen_string_literal: true

require 'forwardable'

module Chemlab
  module Runtime
    # The browser configurator
    class Browser
      extend Forwardable
      attr_accessor :session
      attr_reader :browser_options

      def_delegators :session, :url, :refresh, :text, :quit

      def initialize(browser_options)
        @browser_options = browser_options
        @session = Session.new(browser_options)
      end

      def self.navigate_to(page_class)
        Chemlab.configuration.browser.navigate_to(page_class.path)
      end

      def navigate_to(path)
        @session ||= Chemlab.configuration.browser.session
        @session.engine.goto(Chemlab.configuration.base_url + path)
      end

      # The options used to create the browser session
      def to_s
        @browser_options.to_s
      end

      # An individual session
      class Session
        extend Forwardable
        attr_reader :engine

        def_delegators :engine, :url, :refresh, :text, :quit

        def initialize(browser)
          @engine = Watir::Browser.new(*browser)

          # @engine.goto(Chemlab.configuration.base_url)
        end

        def save_screenshot(file_name)
          engine.screenshot.save(file_name)
        end

        def wait_until(timeout, message = nil, &block)
          engine.wait_until(timeout: timeout, message: message, &block)
        end

        def wait_while(timeout, message = nil, &block)
          engine.wait_while(timeout: timeout, message: message, &block)
        end
      end
    end
  end
end
